/*******************************************************************************
 * Copyright (c) 2000, 2004 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.eclipse.team.internal.target.subscriber;

import org.eclipse.core.resources.*;
import org.eclipse.core.runtime.*;
import org.eclipse.team.core.TeamException;
import org.eclipse.team.internal.core.NullSubProgressMonitor;
import org.eclipse.team.internal.webdav.core.Policy;

/**
 * Operation to get resources from an FTP server.
 */
public class GetOperation extends TargetOperation {

	private boolean overwriteLocalChanges;

	public GetOperation(String deploymentPrivderId, IResource[] resources, int depth, boolean overwriteLocalChanges) {
		super(deploymentPrivderId, resources, depth);
		this.overwriteLocalChanges = overwriteLocalChanges;
	}

	/**
	 * Make the local directories matching the description of the local resource state.
	 */
	private void createLocalFolders(TargetDeploymentProvider provider, IResource resource, IProgressMonitor progress) throws TeamException {	
		try {
			if (!resource.exists()) {
				createLocalFolders(provider, resource.getParent(), progress);// TODO: how to handle progress?
				if (resource.getType() == IResource.FOLDER) {
					byte[] remoteBaseIdentifier = getRemoteSyncBytes(provider, resource);
					((IFolder)resource).create(false /* force */, true /* make local */, progress);
					// Mark the folders as having a base
					getSynchronizer(provider).setBaseBytes(resource, remoteBaseIdentifier);
				}
			}
		} catch (CoreException exception) {
			// The creation failed.
			throw TeamException.asTeamException(exception);
		}
	}
	
	private void download(TargetDeploymentProvider provider, IResource localResource, IProgressMonitor monitor) throws TeamException {
		// Download only applies to files as folders are created by super class
		if (localResource.getType() != IResource.FILE) return;
		
		// Download the file and set the local and remote base identifiers
		final IFile localFile = (IFile)localResource;
		monitor.beginTask(null, 100);
		try {
			// Get the remote base identifier first to ensure that we don't lose remote changes
			byte[] remoteBaseIdentifier = getRemoteSyncBytes(provider, localResource);
			getFile(provider, localFile, Policy.subMonitorFor(monitor, 90));
			// Set the local base identifier (there's always a chance someone snuck in between lines)
			getSynchronizer(provider).setBaseBytes(localResource, remoteBaseIdentifier);
		} finally {
			monitor.done();
		}
	}

	private void getFile(TargetDeploymentProvider provider, IFile localFile, IProgressMonitor monitor) throws TeamException {
		provider.getFile(localFile, monitor);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.team.internal.ftp.deployment.FTPOperation#execute(org.eclipse.team.internal.ftp.client.FTPClient, org.eclipse.core.runtime.IProgressMonitor)
	 */
	protected void execute(TargetDeploymentProvider provider, IResource[] resources, IProgressMonitor monitor) throws TeamException {
		monitor.beginTask(null, 100 * resources.length);
		// TODO: Perform a refresh before performing the get so that the synchronizer contains the remote state
		for (int i = 0; i < resources.length; i++) {
			IResource resource = resources[i];
			get(provider, resource, getDepth(), Policy.subMonitorFor(monitor, 70));
		}
		monitor.done();
	}

	/*
	 * Get the resource (depth = zero).
	 */
	private boolean get(TargetDeploymentProvider provider, IResource resource, IProgressMonitor progress) throws TeamException {
		
		// The no progress monitor is used to control the progress given to
		// other methods and ensures that the progress monitor is not overloaded
		// with subtask messages and work. The null monitor does propagate 
		// cancellation.
		IProgressMonitor noProgress = new NullSubProgressMonitor(progress);
		
		// If remote does not exist then simply ensure no mapped local resource exists.			
		if (!hasRemote(provider, resource)) {
			if (hasMappedLocal(provider, resource)) 
				deleteLocal(provider, resource, noProgress);
			return false; // No need to visit children
		}
		
		// Ensure that the required local folders exist
		if (!resource.exists()) {
			createLocalFolders(provider, resource, noProgress);
		}
		
		// If the remote resource is a file, download the remote contents
		if (resource.getType() == IResource.FILE) {
			download(provider, resource, progress);
			return false; // No need to visit children
		}
		
		// The remote resource is a container.
		
		// If the local resource is a file, we must remove it first.
		if (resource.getType() == IResource.FILE) {
			if (resource.exists()) {
				deleteLocal(provider, resource, noProgress); // May not exist.
			}
			// change the local resource to a folder and create it
			resource = resource.getParent().getFolder(new Path(resource.getName()));
			createLocalFolders(provider, resource, noProgress);
		}
		return true; // Need to visit children (depending on depth)
	}
	
	private void get(TargetDeploymentProvider provider, IResource resource, int depth, IProgressMonitor monitor) throws TeamException {
		IProgressMonitor progress = Policy.monitorFor(monitor);
		try {
			progress.beginTask(null, 100);
			Policy.checkCanceled(progress);
			
			// Perform the get operation of the resource itself
			// and determine if we shoul continue
			if (!get(provider, resource, Policy.subMonitorFor(progress, 100)) || depth == IResource.DEPTH_ZERO) {
				// There is no need to visit the children
				return;
			}
			
			// TODO: may want to exclude uncontrolled locals
			IResource[] children = getSynchronizer(provider).members(resource);
			for (int i = 0; i < children.length; i++) {
				IResource localChild = children[i];
				get(provider, localChild, (depth == IResource.DEPTH_INFINITE) ? IResource.DEPTH_INFINITE : IResource.DEPTH_ONE, Policy.subMonitorFor(progress, 100));
			}
		} finally {
			progress.done();
		}
	}

	private boolean hasMappedLocal(TargetDeploymentProvider provider, IResource resource) throws TeamException {
		return getSynchronizer(provider).hasSyncBytes(resource);
	}

	/**
	 * Delete the local resource represented by the resource state.  Do not complain if the resource does not exist.
	 */
	private final void deleteLocal(TargetDeploymentProvider provider, IResource resource, IProgressMonitor progress) throws TeamException {
		try {
			if (resource.exists()) {
				resource.delete(IResource.KEEP_HISTORY, progress);
			}
			getSynchronizer(provider).flush(resource, IResource.DEPTH_INFINITE);
		} catch (CoreException exception) {
			throw TeamException.asTeamException(exception);
		}
	}
	
}
