/*******************************************************************************
 * Copyright (c) 2000, 2004 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.eclipse.team.internal.target.subscriber;

import java.net.MalformedURLException;
import java.net.URL;

import org.eclipse.core.resources.*;
import org.eclipse.core.runtime.*;
import org.eclipse.team.core.TeamException;
import org.eclipse.team.core.variants.IResourceVariant;
import org.eclipse.team.core.variants.ThreeWaySynchronizer;
import org.eclipse.team.internal.core.Assert;
import org.eclipse.team.internal.core.target.*;
import org.eclipse.team.internal.webdav.core.Policy;

/**
 * Provides common behavior between Target deployment providers
 */
public abstract class TargetDeploymentProvider extends DeploymentProvider {

	// The url that contains the remote folder that is mapped to
	// the container of this deployment provider
	private URL remoteURL;
	
	protected TargetDeploymentProvider() {
		// URL will be set on restore from disk
	}
	
	protected TargetDeploymentProvider(URL remoteUrl) {
		this.remoteURL = remoteUrl;
	}
	
	/**
	 * This method provides a mechanism for allowing a target provider to optimize connections
	 * over a set of operations. The default behavior of the method is to run the provided runnable
	 * and to supply it with the provided progress monitor. Specific target providers may override this
	 * method in order to ensure that the number of connections made to a remote server is minimized.
	 * Calls to this method are nestable and it is up to the target provider to ensure that this is 
	 * supported.
	 */
	public void run(final ITargetRunnable runnable, IProgressMonitor monitor) throws TeamException {
		try {
			ResourcesPlugin.getWorkspace().run(new IWorkspaceRunnable() {
				public void run(IProgressMonitor monitor) throws CoreException {
					try {
						runnable.run(monitor);
					} catch (TeamException e) {
						throw new CoreException(e.getStatus());
					}
				}
			}, getMappedContainer(), 0, monitor);
		} catch (CoreException e) {
			throw new TeamException(e.getStatus());
		}
	}
	
	public abstract void putFile(IFile localFile, IProgressMonitor monitor) throws TeamException;
	
	public abstract void getFile(IFile localFile, IProgressMonitor monitor) throws TeamException;
	
	public abstract void deleteRemote(IResource resource, IProgressMonitor monitor) throws TeamException;
	
	public abstract void createRemoteDirectory(IResource resource, IProgressMonitor monitor) throws TeamException;
	
	public abstract ThreeWaySynchronizer getSynchronizer();

	public abstract IResourceVariant getResourceVariant(IResource resource, byte[] syncBytes) throws TeamException;

	public abstract byte[] fetchRemoteSyncBytes(IResource localResource, IProgressMonitor monitor) throws TeamException;

	public IPath getRelativePath(IResource resource) {
		IPath parentPath = getMappedContainer().getProjectRelativePath();
		IPath resourcePath = resource.getProjectRelativePath();
		if (parentPath.isPrefixOf(resourcePath)) {
			return resourcePath.removeFirstSegments(parentPath.segmentCount());
		} else {
			Assert.isTrue(false, Policy.bind("TargetDeploymentProvider.0", resource.getFullPath().toString(), getMappedContainer().getFullPath().toString())); //$NON-NLS-1$
			return null;
		}
	}

	public URL getUrl(IResource resource) throws MalformedURLException {
		return UrlUtil.concat(getUrl().toExternalForm(), getRelativePath(resource));
	}

	public URL getUrl() {
		return remoteURL;
	}
	
	/**
	 * Method used by subclasses to set the URL when restoring on a restart
	 */
	protected void setUrl(URL url) {
		remoteURL = url;
	}
	
	public static Site getSite(URL url) throws TeamException {
		Site[] sites = TargetManager.getSites();
		for (int i = 0; i < sites.length; i++) {
			Site site = sites[i];
			URL siteUrl = site.getURL();
			// Ensure that we can match URLs to the same host but on different ports
			if (siteUrl.getHost().equals(url.getHost())  && siteUrl.getPort() == url.getPort() && url.getPath().startsWith(siteUrl.getPath())) {
				return site;
			}	
		}
		throw new TeamException(Policy.bind("TargetDeploymentProvider.1", url.toExternalForm()), null); //$NON-NLS-1$
	}

	public void init() {
		getSubscriber().rootAdded(getMappedContainer());
	}

	/**
	 * @return
	 */
	protected abstract TargetSubscriber getSubscriber();

	public void dispose() {
		getSubscriber().rootRemoved(getMappedContainer());
	}

	/**
	 * Return the site associated with the provider.
	 * @return a site
	 */
	public abstract Site getSite();

}
