/*******************************************************************************
 * Copyright (c) 2000, 2004 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.eclipse.team.internal.target.subscriber;

import java.util.*;

import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.*;
import org.eclipse.core.runtime.jobs.ISchedulingRule;
import org.eclipse.team.core.*;
import org.eclipse.team.core.variants.*;
import org.eclipse.team.internal.core.target.*;
import org.eclipse.team.internal.core.target.ITargetRunnable;
import org.eclipse.team.internal.webdav.core.TeamWebDavPlugin;

/**
 * Provide common behavior for get and put target operations
 */
public abstract class TargetOperation {
	
	private int depth;
	private IResource[] resources;
	private String deploymentPrivderId;
	
	protected TargetOperation(String deploymentPrivderId, IResource[] resources, int depth) {
		this.deploymentPrivderId = deploymentPrivderId;
		this.resources = resources;
		this.depth = depth;
	}
	
	/**
	 * Method which can be invoked by clients. It creates  connection.
	 */
	public void run(IProgressMonitor monitor) throws TeamException {
		execute(monitor);
	}
	
	/*
	 * Helper method. Return a Map mapping provider to a list of resources
	 * shared with that provider.
	 */
	private Map getProviderMapping(IResource[] resources) {
		IDeploymentProviderManager deploymentManager = TeamWebDavPlugin.getDeploymentManager();
		Map result = new HashMap();
		for (int i = 0; i < resources.length; i++) {
			IResource resource = resources[i];
			DeploymentProvider[] providers = deploymentManager.getMappings(resource, deploymentPrivderId);
			for (int j = 0; j < providers.length; j++) {
				DeploymentProvider provider = providers[j];
				List list = (List)result.get(provider);
				if (list == null) {
					list = new ArrayList();
					result.put(provider, list);
				}
				list.add(resource);
			}
		}
		return result;
	}
	
	/**
	 * Execute the operation using the given connected client to communicate to the server.
	 * @param client a connected FTP client
	 * @param monitor a progress monitor
	 */
	public void execute(IProgressMonitor monitor) throws TeamException {
		Map table = getProviderMapping(getResources());
		Set keySet = table.keySet();
		monitor.beginTask(null, keySet.size() * 1000);
		Iterator iterator = keySet.iterator();
		while (iterator.hasNext()) {
			IProgressMonitor subMonitor = new SubProgressMonitor(monitor, 1000);
			final TargetDeploymentProvider provider = (TargetDeploymentProvider)iterator.next();
			List list = (List)table.get(provider);
			final IResource[] providerResources = (IResource[])list.toArray(new IResource[list.size()]);
			ISchedulingRule rule = getSchedulingRule(provider);
			try {
				Platform.getJobManager().beginRule(rule, monitor);
				provider.run(new ITargetRunnable() {
					public void run(IProgressMonitor monitor) throws TeamException {
						execute(provider, providerResources, monitor);
					}
				}, subMonitor);
			} finally {
				Platform.getJobManager().endRule(rule);
			}
		}
	}
	
	/**
	 * Execute the operation using the given connected client to communicate to the server.
	 * @param provider2
	 * @param providerResources
	 * @param subMonitor
	 */
	protected abstract void execute(TargetDeploymentProvider provider, IResource[] providerResources, IProgressMonitor subMonitor) throws TeamException;

	/**
	 * Return the scheduling rule to be obtained before work
	 * begins on the given provider. By default, it is the provider's container.
	 * This can be changed by subclasses.
	 * @param provider
	 * @return
	 */
	protected ISchedulingRule getSchedulingRule(DeploymentProvider provider) {
		return provider.getMappedContainer();
	}
	
	/**
	 * @return Returns the depth.
	 */
	public int getDepth() {
		return depth;
	}

	/**
	 * @return Returns the resources.
	 */
	public IResource[] getResources() {
		return resources;
	}

	/**
	 * Used the cached remote state to determine if the remote exists.
	 * This implies that the operation may fail if the cached state is
	 * stale. Performing a refresh should allow the operation to proceed.
	 */
	protected boolean hasRemote(TargetDeploymentProvider provider, IResource localResource) throws TeamException {
		return getSynchronizer(provider).getRemoteBytes(localResource) != null;
	}
	
	protected ThreeWaySynchronizer getSynchronizer(TargetDeploymentProvider provider) {
		return provider.getSynchronizer();
	}

	/**
	 * @param localResource
	 * @return
	 */
	protected byte[] getRemoteSyncBytes(TargetDeploymentProvider provider, IResource localResource) throws TeamException {
		return getSynchronizer(provider).getRemoteBytes(localResource);
	}
	
}
