/*******************************************************************************
 * Copyright (c) 2000, 2004 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.eclipse.team.internal.ui.target.subscriber;

import java.util.ArrayList;
import java.util.Iterator;

import org.eclipse.core.resources.*;
import org.eclipse.core.runtime.*;
import org.eclipse.jface.dialogs.InputDialog;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.viewers.*;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.*;
import org.eclipse.team.internal.core.target.DeploymentProvider;
import org.eclipse.team.internal.ui.Utils;
import org.eclipse.team.internal.ui.target.TargetWizardPage;
import org.eclipse.team.internal.webdav.core.Policy;
import org.eclipse.team.internal.webdav.core.TeamWebDavPlugin;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.model.WorkbenchLabelProvider;
import org.eclipse.ui.part.PageBook;

public class DeploymentContainerSelectionPage extends TargetWizardPage {
	
	private String providerType;
	private IContainer selectedContainer;
	private IContainer initialSelection;
	private boolean newDeployment;
	protected PageBook switcher;
	private TreeViewer localFolderViewer;
	private TreeViewer deploymentProviderViewer;
	
	
	class MappedFolderLabelProvider extends LabelProvider {
		ILabelProvider provider;
		protected MappedFolderLabelProvider(ILabelProvider provider) {
			this.provider = provider;
		}
		public void dispose() {
			super.dispose();
			provider.dispose();
		}
		public String getText(Object element) {
			if (element instanceof IResource) {
				return ((IResource)element).getFullPath().toString();
			}
			return super.getText(element);
		}
		public Image getImage(Object element) {
			return provider.getImage(element);
		}
	}
	
	public DeploymentContainerSelectionPage(String pageName, String title, String description, ImageDescriptor titleImage, String providerType) {
		super(pageName, title, titleImage);
		this.providerType = providerType;
		setDescription(description);
	}
	
	protected boolean hasDeployments() {
		return TeamWebDavPlugin.getDeploymentManager().getDeploymentProviderRoots(providerType).length > 0;
	}
	protected void initializeSelection() {
		if (initialSelection != null) {
			DeploymentProvider[] providers = TeamWebDavPlugin.getDeploymentManager().getMappings(initialSelection, providerType);
			if (providers.length > 0) {
				selectedContainer = providers[0].getMappedContainer();
				newDeployment = false;
			} else {
				selectedContainer = initialSelection;
				newDeployment = true;
			}
		} else {
			newDeployment = !hasDeployments();
		}
	}
	
	protected void createPageBook(Composite composite) {
		switcher = new PageBook(composite, SWT.NULL);
		GridData data = new GridData();
		data.heightHint = 250;
		data.widthHint = 400;
		data.grabExcessVerticalSpace = true;
		data.verticalAlignment = GridData.FILL;
		data.horizontalAlignment = GridData.FILL;
		switcher.setLayoutData(data);
	}
	protected void createDeploymentProviderArea(PageBook composite) {
		deploymentProviderViewer = new TreeViewer(composite, SWT.BORDER);
		DeploymentProviderContentProvider content = new DeploymentProviderContentProvider(providerType);
		deploymentProviderViewer.setContentProvider(content);	
		deploymentProviderViewer.setLabelProvider(getLabelProvider(providerType));
		deploymentProviderViewer.setSorter(new ViewerSorter());
		deploymentProviderViewer.addSelectionChangedListener(
				new ISelectionChangedListener() {
					public void selectionChanged(SelectionChangedEvent event) {
						IStructuredSelection selection = (IStructuredSelection)event.getSelection();
						containerSelectionChanged((IContainer) selection.getFirstElement()); // allow null
					}
				});
	
		// This has to be done after the viewer has been laid out
		deploymentProviderViewer.setInput(ResourcesPlugin.getWorkspace());
		org.eclipse.jface.dialogs.Dialog.applyDialogFont(composite);
	}
	
	protected ILabelProvider getLabelProvider(String id) {
		return new DecoratingLabelProvider(
			new DecoratingLabelProvider(
				new DeploymentProviderLabelProvider(id),
				PlatformUI.getWorkbench().getDecoratorManager().getLabelDecorator()),
			new DeploymentProviderLabelDecorator(id));
	}
	
	protected void createLocalFolderArea(PageBook composite) {
		localFolderViewer = new TreeViewer(composite, SWT.BORDER);
		ContainerContentProvider cp = new ContainerContentProvider();
		cp.showClosedProjects(false /* showClosedProjects */);
		localFolderViewer.setContentProvider(cp);
		localFolderViewer.setLabelProvider(WorkbenchLabelProvider.getDecoratingWorkbenchLabelProvider());
		localFolderViewer.setSorter(new ViewerSorter());
		localFolderViewer.addSelectionChangedListener(
				new ISelectionChangedListener() {
					public void selectionChanged(SelectionChangedEvent event) {
						IStructuredSelection selection = (IStructuredSelection)event.getSelection();
						containerSelectionChanged((IContainer) selection.getFirstElement()); // allow null
					}
				});
		localFolderViewer.addDoubleClickListener(
				new IDoubleClickListener() {
					public void doubleClick(DoubleClickEvent event) {
						ISelection selection = event.getSelection();
						if (selection instanceof IStructuredSelection) {
							Object item = ((IStructuredSelection)selection).getFirstElement();
							if (localFolderViewer.getExpandedState(item))
								localFolderViewer.collapseToLevel(item, 1);
							else
								localFolderViewer.expandToLevel(item, 1);
						}
					}
				});
	
		// This has to be done after the viewer has been laid out
		localFolderViewer.setInput(ResourcesPlugin.getWorkspace());
		
		if (allowNewFolder) {
			Button newFolderButton = new Button(composite, SWT.PUSH);
			newFolderButton.setText("New Folder..."); //$NON-NLS-1$
			newFolderButton.addSelectionListener(new SelectionAdapter() {
				public void widgetSelected(SelectionEvent e) {
					try {
						// assume that only one folder is selected in the folder tree
						IStructuredSelection selection = (IStructuredSelection)localFolderViewer.getSelection();
						Object currentSelection = selection.getFirstElement();
						final IContainer parentFolder = getSelectedFolder(selection);					
	
						IFolder newFolder = promptForFolderName(getShell(), parentFolder);
						if (newFolder == null)
							return;
	
						if (!newFolder.exists()) {
							newFolder.create(false, true, null);
						}
	
						localFolderViewer.refresh(currentSelection);
						localFolderViewer.setExpandedState(currentSelection, true);
						localFolderViewer.setSelection(new StructuredSelection(newFolder));
					} catch (CoreException ex) {
						Utils.handle(ex);
						return;
					}
				}

				private IFolder promptForFolderName(final Shell shell, final IContainer parentFolder) {
					final String[] folderName = new String[] {null};
					shell.getDisplay().syncExec(new Runnable() {
						public void run() {
							InputDialog dialog = new InputDialog(shell, 
								Policy.bind("CreateNewFolderAction.title"),  //$NON-NLS-1$
								Policy.bind("CreateNewFolderAction.message"),  //$NON-NLS-1$
								"", //$NON-NLS-1$
								null);
							if(dialog.open() == InputDialog.OK) {
								folderName[0] = dialog.getValue();
							}
						}
					});
					if(folderName[0] != null) {
						return parentFolder.getFolder(new Path(folderName[0]));
					}
					return null;
				}			
			});
		}
		
		org.eclipse.jface.dialogs.Dialog.applyDialogFont(composite);
	}
	
	private IContainer getSelectedFolder(IStructuredSelection selection) {		
		if (!selection.isEmpty()) {
			Iterator it = selection.iterator();
			while(it.hasNext()) {
				Object o = it.next();
				if(o instanceof IContainer) {
					return ((IContainer)o);
				}
			}
		}
		return null;
	}
	
	private boolean validateContainer() {
		if (selectedContainer == null) {
			setErrorMessage(Policy.bind("DeploymentContainerSelectionPage.4"));  //$NON-NLS-1$
			return false;
		}
		return true;
	}
	/**
	 * The container selection has changed in the
	 * tree view. Update the container name field
	 * value and notify all listeners.
	 */
	public void containerSelectionChanged(IContainer container) {
		selectedContainer = container;
		updateEnablements();
	}
	/**
	 * Returns the currently entered container name.
	 * Null if the field is empty. Note that the
	 * container may not exist yet if the user
	 * entered a new container name in the field.
	 */
	public IPath getContainerFullPath() {
		if (selectedContainer == null)
			return null;
		else
			return selectedContainer.getFullPath();
	}
	public IContainer getSelectedContainer() {
		return selectedContainer;
	}
	/**
	 * Sets the selected existing container.
	 */
	protected void setSelectedContainer(IContainer container) {
		selectedContainer = container;
		
		//expand to and select the specified container
		java.util.List itemsToExpand = new ArrayList();
		IContainer parent = container.getParent();
		while (parent != null) {
			itemsToExpand.add(0,parent);
			parent = parent.getParent();
		}
		setSelection(getVisibleViewer(), container, itemsToExpand.toArray());
	}
	protected TreeViewer getVisibleViewer() {
		if (isNewDeployment()) {
			return localFolderViewer;
		} else {
			return deploymentProviderViewer;
		}
	}
	/**
	 * @param container
	 * @param itemsToExpand
	 */
	private void setSelection(TreeViewer treeViewer, IContainer container, Object[] itemsToExpand) {
		treeViewer.setExpandedElements(itemsToExpand); 
		treeViewer.setSelection(new StructuredSelection(container),true);
	}
	private void updateEnablements() {
		if (!validateContainer()) {
			setPageComplete(false);
			return;
		}
		setErrorMessage(null);
		setPageComplete(true);
	}
	/**
	 * Toggles between folder/provider view
	 */
	protected void switchViewerType() {
		selectedContainer = null;
		switcher.showPage(getVisibleViewer().getControl());
		getVisibleViewer().setSelection(new StructuredSelection());
		updateEnablements();
	}
	public boolean isNewDeployment() {
		return newDeployment;
	}
	/**
	 * Set the initial selection for the page
	 * @param selection the initial selection
	 */
	public void setInitialSelection(IStructuredSelection selection) {
		if (selection != null) {
			Object element = selection.getFirstElement();
			if (element instanceof IFile) {
				initialSelection = ((IFile) element).getParent();
			} else if (element instanceof IContainer) {
				initialSelection = (IContainer) element;
			}
		}
	}
	private Button useExistingDeployment;
	private Button createNewDeployment;
	private boolean allowNewFolder;
	public void createControl(Composite parent) {
		Composite composite = createComposite(parent, 1);
		// set F1 help
		// TODO: WorkbenchHelp.setHelp(composite, IHelpContextIds.TARGET_NEW_SITE_PAGE);
		setControl(composite);
			
		initializeSelection();
		
		// Create deployment method selection buttons
		useExistingDeployment = createRadioButton(composite, getExistingDeploymentLabel(), 1); //$NON-NLS-1$
		createNewDeployment = createRadioButton(composite, getCreateNewDeploymentLabel(), 1); //$NON-NLS-1$
		useExistingDeployment.setSelection(!newDeployment);
		createNewDeployment.setSelection(newDeployment);
		
		createPageBook(composite);
		createLocalFolderArea(switcher);
		createDeploymentProviderArea(switcher);
		org.eclipse.jface.dialogs.Dialog.applyDialogFont(parent);
	
		// Register button listeners
		SelectionAdapter listener = new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				if (createNewDeployment.getSelection() != newDeployment) {
					newDeployment = createNewDeployment.getSelection();
					switchViewerType();
				}
			}
		};
		useExistingDeployment.addSelectionListener(listener);
		createNewDeployment.addSelectionListener(listener);
		
		if (selectedContainer == null) {
			setPageComplete(false);
		} else {
			setSelectedContainer(selectedContainer);
			setPageComplete(true);
		}
		switcher.showPage(getVisibleViewer().getControl());
	}

	/**
	 * @return
	 */
	protected String getCreateNewDeploymentLabel() {
		return Policy.bind("DeploymentContainerSelectionPage.5"); //$NON-NLS-1$
	}

	/**
	 * @return
	 */
	protected String getExistingDeploymentLabel() {
		return Policy.bind("DeploymentContainerSelectionPage.6"); //$NON-NLS-1$
	}

	public void allowNewFolder(boolean b) {
		this.allowNewFolder = b;
	}
}
