/*******************************************************************************
 * Copyright (c) 2000, 2004 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.eclipse.team.internal.ui.target.subscriber;

import java.lang.reflect.InvocationTargetException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.List;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.IPath;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.team.core.TeamException;
import org.eclipse.team.internal.core.target.*;
import org.eclipse.team.internal.target.subscriber.TargetDeploymentProvider;
import org.eclipse.team.internal.target.subscriber.TargetSubscriber;
import org.eclipse.team.internal.ui.ITeamUIImages;
import org.eclipse.team.internal.ui.TeamUIPlugin;
import org.eclipse.team.internal.webdav.core.Policy;
import org.eclipse.team.internal.webdav.core.TeamWebDavPlugin;
import org.eclipse.team.ui.synchronize.*;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchWizard;
import org.eclipse.ui.ide.IDE;

public abstract class DeploymentWizard extends Wizard implements IWorkbenchWizard, ExportResourceSelectionPage.IDeploymentPageInput {

	private IWorkbench workbench;
	protected IStructuredSelection selection;
	private SubscriberParticipant participant;

	public DeploymentWizard() {
		initializeDialogSettings();
	}
	
	public void init(IWorkbench workbench, IStructuredSelection currentSelection) {
		this.workbench = workbench;
		this.selection = currentSelection;
	
		List selectedResources = IDE.computeSelectedResources(currentSelection);
		if (!selectedResources.isEmpty()) {
			this.selection = new StructuredSelection(selectedResources);
		}
	
		setDefaultPageImageDescriptor(TeamUIPlugin.getImageDescriptor(ITeamUIImages.IMG_WIZBAN_SHARE));
		setNeedsProgressMonitor(true);
	}

	protected void handle(InvocationTargetException e) {
		handle(e.getTargetException());
	}

	protected void handle(Throwable e) {
		new ErrorHandler(TeamWebDavPlugin.getPlugin()).openError(getShell(), null, null, e, ErrorHandler.LOG_OTHER_EXCEPTIONS);
	}

	protected TargetDeploymentProvider createdDeploymentProvider;

	public TargetDeploymentProvider getSelectedDeploymentProvider() {
		// If there was a previously create provider, unmap it
		if (createdDeploymentProvider != null) {
			try {
				TeamWebDavPlugin.getDeploymentManager().unmap(createdDeploymentProvider.getMappedContainer(), createdDeploymentProvider);
			} catch (TeamException e) {
				TeamWebDavPlugin.log(e);
			} finally {
				createdDeploymentProvider = null;
			}
		}
		if (isNewDeployment()) {
			try {
				Site site = getSite();
				ensureSiteRegistered(site);
				URL url = site.getURL();
				IPath path = getMapping();
				url = UrlUtil.concat(url.toExternalForm(), path);
				IContainer container = getSelectedContainer();
				ensureUnmapped(container);
				createdDeploymentProvider = createDeploymentProvider(url);
				TeamWebDavPlugin.getDeploymentManager().map(container, createdDeploymentProvider);
				setBaseBytes(container, createdDeploymentProvider);
				return createdDeploymentProvider;
			} catch (TeamException e) {
				handle(e);
				return null;
			} catch (MalformedURLException e) {
				handle(e);
				return null;
			}
		} else {
			DeploymentProvider[] providers = TeamWebDavPlugin.getDeploymentManager().getMappings(getSelectedContainer(), getDeploymentProviderId());
			return (TargetDeploymentProvider)providers[0];
		}
	}

	protected abstract IPath getMapping();

	protected abstract IContainer getSelectedContainer();

	protected abstract boolean isNewDeployment();

	protected abstract Site getSite();

	private void ensureSiteRegistered(Site site) {
		Site[] sites = TargetManager.getSites();
		for (int i = 0; i < sites.length; i++) {
			Site existingSite = sites[i];
			if (existingSite.equals(site)) {
				return;
			}
		}
		TargetManager.addSite(site);
	}

	public boolean performCancel() {
		if (createdDeploymentProvider != null) {
			try {
				if (!MessageDialog.openQuestion(getShell(),
						Policy.bind("DeploymentWizard.0"), //$NON-NLS-1$
						Policy.bind("DeploymentWizard.1"))) { //$NON-NLS-1$
					TeamWebDavPlugin.getDeploymentManager().unmap(createdDeploymentProvider.getMappedContainer(), createdDeploymentProvider);
				}
				
			} catch (TeamException e) {
				TeamWebDavPlugin.log(e);
			}
			createdDeploymentProvider = null;
		}
		return super.performCancel();
	}

	protected abstract void initializeDialogSettings();

	public abstract TargetSubscriber getSubscriber();

	protected abstract String getSiteType();

	public abstract String getDeploymentProviderId();

	protected abstract void setBaseBytes(IContainer container, TargetDeploymentProvider provider) throws TeamException;

	protected abstract TargetDeploymentProvider createDeploymentProvider(URL url) throws TeamException;

	private void ensureUnmapped(IContainer container) throws TeamException {
		DeploymentProvider[] providers = TeamWebDavPlugin.getDeploymentManager().getMappings(container, getDeploymentProviderId());
		for (int i = 0; i < providers.length; i++) {
			DeploymentProvider provider = providers[i];
			TeamWebDavPlugin.getDeploymentManager().unmap(provider.getMappedContainer(), provider);
		}
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.team.internal.ui.target.subscriber.ExportResourceSelectionPage.IDeploymentPageInput#getParticipant()
	 */
	public ISynchronizeParticipant getParticipant() {
		if (participant == null) {
			participant = createParticipant(new ResourceScope(new IResource[0]));
		}
		return participant;
	}

	protected abstract SubscriberParticipant createParticipant(ISynchronizeScope scope);
}
