/*******************************************************************************
 * Copyright (c) 2000, 2004 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.eclipse.team.internal.ui.target.subscriber;

import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;

import org.eclipse.compare.CompareConfiguration;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.jface.dialogs.ErrorDialog;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.viewers.CheckStateChangedEvent;
import org.eclipse.jface.viewers.CheckboxTreeViewer;
import org.eclipse.jface.viewers.ICheckStateListener;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.team.core.TeamException;
import org.eclipse.team.internal.target.subscriber.TargetDeploymentProvider;
import org.eclipse.team.internal.ui.Utils;
import org.eclipse.team.internal.ui.synchronize.SynchronizePageConfiguration;
import org.eclipse.team.internal.ui.target.TargetWizardPage;
import org.eclipse.team.internal.webdav.core.Policy;
import org.eclipse.team.internal.webdav.core.TeamWebDavPlugin;
import org.eclipse.team.ui.synchronize.ISynchronizePage;
import org.eclipse.team.ui.synchronize.ISynchronizePageConfiguration;
import org.eclipse.team.ui.synchronize.ISynchronizeParticipant;
import org.eclipse.team.ui.synchronize.ParticipantPageSaveablePart;
import org.eclipse.team.ui.synchronize.ResourceScope;
import org.eclipse.team.ui.synchronize.SubscriberParticipant;

public class ExportResourceSelectionPage extends TargetWizardPage {

	private ISynchronizePageConfiguration configuration;
	private IDeploymentPageInput wizard;
	private ParticipantPageSaveablePart input;
	
	public interface IDeploymentPageInput {
		String getDeploymentProviderId();
		TargetDeploymentProvider getSelectedDeploymentProvider();
		ISynchronizeParticipant getParticipant();
	}
	
	public ExportResourceSelectionPage(String pageName, String title, String description, ImageDescriptor titleImage, IDeploymentPageInput wizard) {
		super(pageName, title, titleImage);
		this.wizard = wizard;
		setDescription(description);
	}

	private ParticipantPageSaveablePart createCompareInput() {	
		ISynchronizeParticipant participant = wizard.getParticipant();
		configuration = participant.createPageConfiguration();
		configuration.setMode(ISynchronizePageConfiguration.BOTH_MODE);
		configuration.setMenuGroups(ISynchronizePageConfiguration.P_TOOLBAR_MENU, new String[] { 
				ISynchronizePageConfiguration.SYNCHRONIZE_GROUP, 
				ISynchronizePageConfiguration.NAVIGATE_GROUP,  
				ISynchronizePageConfiguration.LAYOUT_GROUP });
		configuration.setMenuGroups(ISynchronizePageConfiguration.P_CONTEXT_MENU, new String[0]);
		((SynchronizePageConfiguration)configuration).setViewerStyle(SynchronizePageConfiguration.CHECKBOX);
		configuration.addLabelDecorator(new DeploymentProviderLabelDecorator(
						ExportResourceSelectionPage.this.wizard.getDeploymentProviderId()));
		configuration.setRunnableContext(getContainer());
		
		CompareConfiguration cc = new CompareConfiguration();
		cc.setLeftEditable(false);
		cc.setRightEditable(false);
		ParticipantPageSaveablePart part = new ParticipantPageSaveablePart(getShell(), cc, configuration, participant);
		
		setPageComplete(false);
		
		return part;
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.dialogs.IDialogPage#createControl(org.eclipse.swt.widgets.Composite)
	 */
	public void createControl(Composite parent2) {
		Composite parent = new Composite(parent2, SWT.NULL);
		GridLayout layout = new GridLayout();
		layout.marginHeight = 0;
		layout.marginWidth = 0;
		layout.verticalSpacing = 0;
		GridData data = new GridData(GridData.FILL_BOTH);
		data.grabExcessHorizontalSpace = true;
		parent.setLayout(layout);
		parent.setLayoutData(data);
		input = createCompareInput();
		input.createPartControl(parent);
		setControl(parent);
		
		ISynchronizePage page = configuration.getPage();
		if (page != null) {
			Viewer viewer = page.getViewer();
			if (viewer instanceof CheckboxTreeViewer) {
				((CheckboxTreeViewer)viewer).addCheckStateListener(new ICheckStateListener() {
					public void checkStateChanged(CheckStateChangedEvent event) {
						setPageComplete(getSelectedResources().length > 0);
					}
				});
			}
		}
	}

	/* private */ void populatePage(IResource root, IProgressMonitor monitor) throws TeamException {
		try {
			String taskName = Policy.bind("ExportResourceSelectionPage.1", root.getFullPath().toString()); //$NON-NLS-1$
			monitor.beginTask(taskName, 100); //$NON-NLS-1$
			IResource[] roots;
			if (root == null) {
				roots = new IResource[0];
			} else {
				roots = new IResource[] { root };
			}
			((ResourceScope)((SubscriberParticipant)configuration.getParticipant()).getScope()).setResources(roots);
			((SubscriberParticipant)configuration.getParticipant()).refreshNow(roots, taskName, monitor);
		} finally {
			monitor.done();
		}
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.dialogs.IDialogPage#setVisible(boolean)
	 */
	public void setVisible(boolean visible) {
		super.setVisible(visible);
		if (visible) {
			// Get the provider from the wizard (creating it if required)
			TargetDeploymentProvider provider = wizard.getSelectedDeploymentProvider();
			final IResource root;
			if (provider == null) {
				root = null;
			} else {
				root = provider.getMappedContainer();
			}
			try {
				getContainer().run(true, true, new IRunnableWithProgress() {
					public void run(final IProgressMonitor monitor) throws InvocationTargetException, InterruptedException {
						try {
							populatePage(root, monitor);
						} catch (TeamException e) {
							throw new InvocationTargetException(e);
						}
					}
				});
			} catch (InvocationTargetException e) {
				handleError(e);
			} catch (InterruptedException e) {
				// TODO: Ignore for now
			}
		}
	}

	private void handleError(InvocationTargetException e) {
		IStatus status = null;
		String message;
		if (e.getTargetException() instanceof CoreException) {
			status = ((CoreException)e.getTargetException()).getStatus();
			message = Policy.bind("ExportResourceSelectionPage.2"); //$NON-NLS-1$
		} else {
			status = null;
			message = Policy.bind("ExportResourceSelectionPage.3"); //$NON-NLS-1$
			TeamWebDavPlugin.log(IStatus.ERROR, message, e.getTargetException());
		}
		setPageComplete(false);
		ErrorDialog.openError(getShell(), Policy.bind("ExportResourceSelectionPage.4"), message, status); //$NON-NLS-1$
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.dialogs.IDialogPage#dispose()
	 */
	public void dispose() {
		input.dispose();
		super.dispose();
	}

	public IResource[] getSelectedResources() {
		ISynchronizePage page = configuration.getPage();
		if (page != null) {
			Viewer viewer = page.getViewer();
			if (viewer instanceof CheckboxTreeViewer) {
				Object[] elements = ((CheckboxTreeViewer)viewer).getCheckedElements();
				IResource[] resources = Utils.getResources(elements);
				ArrayList result = new ArrayList();
				for (int i = 0; i < resources.length; i++) {
					IResource resource = resources[i];
					if (configuration.getSyncInfoSet().getSyncInfo(resource) != null) {
						result.add(resource);
					}
				}
				return (IResource[]) result.toArray(new IResource[result.size()]);
			}
		}
		return new IResource[0];
	}
}
