/*******************************************************************************
 * Copyright (c) 2000, 2004 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.eclipse.team.internal.ui.target.subscriber;

import java.lang.reflect.InvocationTargetException;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.*;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.jface.wizard.IWizardPage;
import org.eclipse.team.core.TeamException;
import org.eclipse.team.internal.core.target.Site;
import org.eclipse.team.internal.target.subscriber.TargetSubscriber;
import org.eclipse.team.internal.ui.ITeamUIImages;
import org.eclipse.team.internal.ui.TeamUIPlugin;
import org.eclipse.team.internal.webdav.core.Policy;

public abstract class ImportWizard extends DeploymentWizard {
	
	private SiteCreationPage siteCreationPage;
	private SiteSelectionPage siteSelectionPage;
	private ImportTargetSelectionPage containerSelectionPage;
	private MappingSelectionPage mappingPage;
	private ImportResourceSelectionPage resourceSelectionPage;
	
	public void addPages() {
		containerSelectionPage = new ImportTargetSelectionPage(
				"container-selection",  //$NON-NLS-1$
				Policy.bind("ImportWizard.1"),  //$NON-NLS-1$
				Policy.bind("ImportWizard.2"),  //$NON-NLS-1$
				TeamUIPlugin.getImageDescriptor(ITeamUIImages.IMG_WIZBAN_SHARE),
				getDeploymentProviderId());
		//siteSelectionPage.setDialogSettings(getDialogSettings());
		containerSelectionPage.setInitialSelection(selection);
		containerSelectionPage.allowNewFolder(true);
		addPage(containerSelectionPage);
		
		siteSelectionPage = new SiteSelectionPage(
				"site-selection",  //$NON-NLS-1$
				Policy.bind("ImportWizard.4"),  //$NON-NLS-1$
				Policy.bind("ImportWizard.5"),  //$NON-NLS-1$
				TeamUIPlugin.getImageDescriptor(ITeamUIImages.IMG_WIZBAN_SHARE), 
				getSiteType());
		//siteSelectionPage.setDialogSettings(getDialogSettings());
		addPage(siteSelectionPage);
		
		siteCreationPage = createSiteCreationPage();
		siteCreationPage.setDialogSettings(getDialogSettings());
		addPage(siteCreationPage);
		
		mappingPage = new MappingSelectionPage(
				"site-mapping",  //$NON-NLS-1$
				Policy.bind("ImportWizard.7"), //$NON-NLS-1$
				Policy.bind("ImportWizard.8"), //$NON-NLS-1$
				TeamUIPlugin.getImageDescriptor(ITeamUIImages.IMG_WIZBAN_SHARE));
		//mappingPage.setDialogSettings(getDialogSettings());
		mappingPage.setAllowNewFolder(false);
		addPage(mappingPage);
		
		resourceSelectionPage = new ImportResourceSelectionPage(
				"resource-selection", //$NON-NLS-1$
				Policy.bind("ImportWizard.10"), //$NON-NLS-1$
				Policy.bind("ImportWizard.11"), //$NON-NLS-1$
				TeamUIPlugin.getImageDescriptor(ITeamUIImages.IMG_WIZBAN_SHARE),
				this);
		addPage(resourceSelectionPage);
	}
	
	public abstract TargetSubscriber getSubscriber();

	protected abstract SiteCreationPage createSiteCreationPage();

	protected abstract String getSiteType();

	public abstract String getDeploymentProviderId();

	public IWizardPage getNextPage(IWizardPage page) {
		if (page == containerSelectionPage) {
			if (containerSelectionPage.isNewDeployment()) {
				return siteSelectionPage;
			} else {
				return resourceSelectionPage;
			}
		}
		if (page == siteSelectionPage) {
			if(siteSelectionPage.getSite() != null) {
				mappingPage.setSite(siteSelectionPage.getSite());
				mappingPage.setContainer(containerSelectionPage.getSelectedContainer());
				mappingPage.setPreviousPage(page);
				return mappingPage;
			} else {
				return siteCreationPage;
			}
		}
		if (page == siteCreationPage) {
			mappingPage.setSite(siteCreationPage.getSite());
			mappingPage.setContainer(containerSelectionPage.getSelectedContainer());
			mappingPage.setPreviousPage(page);
			return mappingPage;
		}
		if (page == mappingPage) {
			return resourceSelectionPage;
		}
		return super.getNextPage(page);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.wizard.IWizard#performFinish()
	 */
	public boolean performFinish() {
		// Make sure that the deployment provider has been created if needed
		if (createdDeploymentProvider == null) {
			// This may or may not create a deployment provider.
			getSelectedDeploymentProvider();
		}
 		try {
 			final IResource[] selected = resourceSelectionPage.getSelectedResources();
			getContainer().run(true, true, new IRunnableWithProgress() {
				public void run(IProgressMonitor monitor) throws InvocationTargetException, InterruptedException {
					try {
						IStatus status = getSubscriber().getTeamOperations().get(selected, IResource.DEPTH_INFINITE, true, monitor);
						if (!status.isOK()) {
							throw new TeamException(status);
						}
					} catch (TeamException e) {
						throw new InvocationTargetException(e);
					}
				}
			});
		} catch (InvocationTargetException e) {
			handle(e);
		} catch (InterruptedException e) {
			return false;
		}
		return true;
	}

	protected Site getSite() {
		Site site = siteSelectionPage.getSite();
		if (site == null) {
			site = siteCreationPage.getSite();
		}
		return site;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.wizard.IWizard#canFinish()
	 */
	public boolean canFinish() {
		if (containerSelectionPage.isPageComplete() &&
				siteSelectionPage.isPageComplete() &&
				mappingPage.isPageComplete() &&
				getSite() != null &&
				resourceSelectionPage.isPageComplete()) {
			return true;
		}
		return super.canFinish();
	}

	protected boolean isNewDeployment() {
		return containerSelectionPage.isNewDeployment();
	}
	
	protected IContainer getSelectedContainer() {
		return containerSelectionPage.getSelectedContainer();
	}
	
	protected IPath getMapping() {
		return mappingPage.getMapping();
	}
}
