/*******************************************************************************
 * Copyright (c) 2000, 2004 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.eclipse.team.internal.webdav.subscriber;

import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URL;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.team.core.TeamException;
import org.eclipse.team.core.variants.IResourceVariant;
import org.eclipse.team.core.variants.ThreeWaySynchronizer;
import org.eclipse.team.internal.core.streams.PollingInputStream;
import org.eclipse.team.internal.core.streams.ProgressMonitorInputStream;
import org.eclipse.team.internal.core.target.IMemento;
import org.eclipse.team.internal.core.target.Site;
import org.eclipse.team.internal.core.target.UrlUtil;
import org.eclipse.team.internal.target.subscriber.TargetDeploymentProvider;
import org.eclipse.team.internal.target.subscriber.TargetSubscriber;
import org.eclipse.team.internal.webdav.core.Authenticator;
import org.eclipse.team.internal.webdav.core.Policy;
import org.eclipse.team.internal.webdav.core.TeamWebDavPlugin;
import org.eclipse.team.internal.webdav.core.WebDavSite;
import org.eclipse.webdav.ILocator;
import org.eclipse.webdav.client.DAVClient;
import org.eclipse.webdav.client.RemoteDAVClient;
import org.eclipse.webdav.client.ResourceHandle;
import org.eclipse.webdav.client.WebDAVFactory;
import org.eclipse.webdav.http.client.HttpClient;
import org.eclipse.webdav.internal.kernel.DAVException;
import org.eclipse.webdav.internal.kernel.WebDAVException;

public class WebDavDeploymentProvider extends TargetDeploymentProvider {

	public static final int TRANSFER_PROGRESS_INCREMENT = 32768;
	
	private final String CTX_URL_KEY = "URL"; //$NON-NLS-1$
	
	/*
	 * The WebDAVFactory is used to create new resource handles, locators, and
	 * contexts, etc.
	 */
	protected static WebDAVFactory DAV_FACTORY = new WebDAVFactory();
	
	// Re-use the http client
	private static HttpClient httpClient = new HttpClient();
	
	// The url that contains the remote folder that is mapped to
	// the container of this deployment provider
	private URL remoteURL;

	private DAVClient davClient;
	
	/*
	 * Constructor required for creation by the Team deployment provider manager
	 */
	public WebDavDeploymentProvider() {
		// Url will be set in restoreState
	}
	
	public WebDavDeploymentProvider(URL url) throws TeamException {
		remoteURL = url;
	}
	
	public String getID() {
		return TeamWebDavPlugin.DEPLOYMENT_PROVIDER_ID;
	}
	
	private DAVClient initializeDavClient() throws TeamException {
		WebDavSite site = getWebDavSite();

		// Authentication.
		String username = site.getUsername();
		String password = site.getPassword();
		if ((username != null) && (password != null))
			httpClient.setAuthenticator(new Authenticator(username, password));

		// Proxy server.
		String proxy = site.getProxy();
		if (proxy != null && proxy.length() > 0) {
			try {
				httpClient.setDefaultProxyServerUrl(new URL(proxy));
			} catch (MalformedURLException exception) {
				// Setting the proxy fails if the URL is invalid.
				throw new TeamException(Policy.bind("Config.error"), exception); //$NON-NLS-1$
			}
		}

		// Timeout converted to miliseconds.
		int timeout = site.getTimeout() * 1000;
		// connection recycler timeout and socket timeout being set to the same
		// value.
		httpClient.setConnectionTimeout(timeout);
		httpClient.setSoTimeout(timeout);

		// Can add other configs here later.
		return new RemoteDAVClient(new WebDAVFactory(), httpClient);
	}
	/* (non-Javadoc)
	 * @see org.eclipse.team.core.DeploymentProvider#saveState(org.eclipse.team.core.IMemento)
	 */
	public void saveState(IMemento memento) {
		memento.putString(CTX_URL_KEY, remoteURL.toExternalForm());
//		 TODO: Need to persist site (or enough info to recreate the site)
	}

	/* (non-Javadoc)
	 * @see org.eclipse.team.core.DeploymentProvider#restoreState(org.eclipse.team.core.IMemento)
	 */
	public void restoreState(IMemento memento) {
		String urlString = memento.getString(CTX_URL_KEY);
		if (urlString == null) {
			handleStateError(null);
			return;
		}
		try {
			remoteURL = new URL(memento.getString(CTX_URL_KEY));
		} catch (MalformedURLException e) {
			handleStateError(e);
		}
	}

	private void handleStateError(Throwable e) {
		// invalid state. 
		// Log it and remove the provider
		TeamWebDavPlugin.logError(Policy.bind("WebDavDeploymentProvider.5", getMappedContainer().getFullPath().toString()), e); //$NON-NLS-1$
//		try {
//			Team.getDeploymentManager().unmap(getMappedContainer(), this);
//		} catch (TeamException e1) {
//			FTPPlugin.log(e1);
//		}
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.team.core.target.TargetDeploymentProvider#putFile(org.eclipse.core.resources.IFile, org.eclipse.core.runtime.IProgressMonitor)
	 */
	public void putFile(IFile localFile, IProgressMonitor monitor) throws TeamException {
		try {
			// Copy from the local file to the remote resource.
			InputStream in = null;
			try {
				in = localFile.getContents();
				long size = localFile.getLocation().toFile().length();
				
				// To be precise we should set the If-Match: header to ensure we do not overwrite others' changes.
				setContent(getUrl(localFile), size, in, monitor);
			} finally {
				if (in != null)
					in.close();
			}
		} catch (IOException exception) {
			throw TeamWebDavPlugin.wrapException(exception);
		} catch (CoreException exception) {
			throw TeamWebDavPlugin.wrapException(exception);
		} catch(DAVException exception) {
			throw TeamWebDavPlugin.wrapException(exception);
		}
	}

	private void setContent(URL url, final long size, InputStream is, IProgressMonitor progress) throws DAVException, TeamException {
		ResourceHandle davHandle = getDavResource(url);
		final String name = UrlUtil.toTruncatedPath(url.toExternalForm(), 3);
		
		// timeout value for this site
		int timeout = getWebDavSite().getTimeout();

		final String prefix = Policy.bind("DavRemoteTargetResource.upload", name); //$NON-NLS-1$
		progress.subTask(Policy.bind("DavRemoteTargetResource.transferNoSize", prefix)); //$NON-NLS-1$
		
		// wrap the dav input stream with an input stream that allows progress
		// and cancellation.
		InputStream in = new ProgressMonitorInputStream(
				new PollingInputStream(is, 
						timeout,
						progress), 
						0, 
						TRANSFER_PROGRESS_INCREMENT, 
						progress) {
			// perform updates to the progress message, showing the number of bytes transfered
			protected void updateMonitor(long bytesRead, long bytesTotal, IProgressMonitor monitor) {
				if (bytesRead == 0) return;
				monitor.subTask(Policy.bind("DavRemoteTargetResource.transfer", //$NON-NLS-1$
						new Object[] { prefix, Long.toString(bytesRead >> 10), Long.toString(size >> 10) }));
			}
		};
		davHandle.setContent(in);
	}
	
	private ResourceHandle getDavResource(URL url) throws TeamException {
		// number of bytes expected
		ILocator locator = DAV_FACTORY.newLocator(url.toExternalForm());
		ResourceHandle davHandle = new ResourceHandle(getDavClient(), locator);
		return davHandle;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.team.core.target.TargetDeploymentProvider#getFile(org.eclipse.core.resources.IFile, org.eclipse.core.runtime.IProgressMonitor)
	 */
	public void getFile(IFile localFile, IProgressMonitor monitor) throws TeamException {
		try {
			InputStream source = null;
			try {
				// Get the remote file content.
				source = getContents(localFile, monitor);
				// Set the local file content to be the same as the remote file.
				if (localFile.exists())
					localFile.setContents(source, false, true, monitor);
				else
					localFile.create(source, false, monitor);
			} finally {
				if (source != null)
					source.close();
			}
		} catch (CoreException exception) {
			throw TeamWebDavPlugin.wrapException(exception);
		} catch (IOException exception) {
			throw TeamWebDavPlugin.wrapException(exception);
		}
	}

	private InputStream getContents(IFile localFile, IProgressMonitor monitor) throws TeamException {
		try {
			return getSubscriberResource(localFile).getStorage(monitor).getContents();
		} catch (CoreException e) {
			throw TeamException.asTeamException(e);
		}
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.team.core.target.TargetDeploymentProvider#deleteRemote(org.eclipse.core.resources.IResource, org.eclipse.core.runtime.IProgressMonitor)
	 */
	public void deleteRemote(IResource resource, IProgressMonitor monitor) throws TeamException {
		try {
			getDavResource(getUrl(resource)).delete();
		} catch (WebDAVException exception) {
			throw TeamWebDavPlugin.wrapException(exception);
		} catch (DAVException exception) {
			throw TeamWebDavPlugin.wrapException(exception);
		} catch (MalformedURLException exception) {
			throw TeamWebDavPlugin.wrapException(exception);
		}
	}

	/* (non-Javadoc)
	 * @see org.eclipse.team.core.target.TargetDeploymentProvider#createRemoteDirectory(org.eclipse.core.resources.IResource, org.eclipse.core.runtime.IProgressMonitor)
	 */
	public void createRemoteDirectory(
		IResource resource,
		IProgressMonitor monitor)
		throws TeamException {
		try {
			getDavResource(getUrl(resource)).asCollectionHandle().mkdirs();
		} catch (DAVException e) {
			throw TeamWebDavPlugin.wrapException(e);
		} catch (MalformedURLException e) {
			throw TeamWebDavPlugin.wrapException(e);
		}

	}

	/* (non-Javadoc)
	 * @see org.eclipse.team.core.target.TargetDeploymentProvider#getSubscriberResource(org.eclipse.core.resources.IResource, byte[])
	 */
	public IResourceVariant getResourceVariant(IResource resource, byte[] syncBytes) throws TeamException {
		try {
			return new WebDavSubscriberResource(getDavResource(getUrl(resource)), resource.getType() != IResource.FILE, syncBytes);
		} catch (MalformedURLException e) {
			throw TeamWebDavPlugin.wrapException(e);
		}
	}
	
	public WebDavSubscriberResource getSubscriberResource(IResource resource) throws TeamException {
		try {
			return WebDavSubscriberResource.create(getDavResource(getUrl(resource)), resource.getType() != IResource.FILE);
		} catch (MalformedURLException e) {
			throw TeamWebDavPlugin.wrapException(e);
		}
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.team.core.target.TargetDeploymentProvider#getSynchronizer()
	 */
	public ThreeWaySynchronizer getSynchronizer() {
		return TeamWebDavPlugin.getPlugin().getSubscriber().getSynchronizer();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.team.core.target.TargetDeploymentProvider#getUrl()
	 */
	public URL getUrl() {
		return remoteURL;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.team.core.target.TargetDeploymentProvider#fetchRemoteSyncBytes(org.eclipse.core.resources.IResource, org.eclipse.core.runtime.IProgressMonitor)
	 */
	public byte[] fetchRemoteSyncBytes(IResource localResource, IProgressMonitor monitor) throws TeamException {
		try {
			return WebDavSubscriberResource.getRemoteSyncBytes(getDavResource(getUrl(localResource)));
		} catch (MalformedURLException e) {
			throw TeamWebDavPlugin.wrapException(e);
		}
	}

	protected WebDavSite getWebDavSite() throws TeamException {
		Site site = getSite(getUrl());
		if (site instanceof WebDavSite) {
			return (WebDavSite)site;
		}
		throw new TeamException(Policy.bind("WebDavDeploymentProvider.6", getUrl().toExternalForm()), null); //$NON-NLS-1$
	}
	
	protected synchronized DAVClient getDavClient() throws TeamException {
		if (davClient == null) {
			davClient = initializeDavClient();
		}
		return davClient;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.team.internal.target.subscriber.TargetDeploymentProvider#getSubscriber()
	 */
	protected TargetSubscriber getSubscriber() {
		return TeamWebDavPlugin.getPlugin().getSubscriber();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.team.internal.target.subscriber.TargetDeploymentProvider#getSite()
	 */
	public Site getSite() {
		try {
			return getWebDavSite();
		} catch (TeamException e) {
			TeamWebDavPlugin.log(e);
			return null;
		}
	}

}
